module ReVIEW
	module BuilderOverride
		# bchapref: 太字＋章番号とタイトル参照
		Compiler.definline :bchapref
		def inline_bchapref(s)
			title = inline_chapref(s)
			if @book.config['chapterlink']
			  "\\hyperref[chap:#{s}]{\\textbf{#{title}}}"
			else
			  inline_b(title)
			end
		  rescue KeyError
			error "unknown chapter: #{s}"
			nofunc_text("[UnknownChapter:#{s}]")
		end

		# btitle: 太字＋タイトル参照
		Compiler.definline :btitle
		def inline_btitle(id)
			title = inline_title(id)
			if @book.config['chapterlink']
			  "\\hyperref[chap:#{id}]{\\textbf{#{title}}}"
			else
			  inline_b(title)
			end
		rescue KeyError
			error "unknown chapter: #{id}"
			nofunc_text("[UnknownChapter:#{id}]")
		end		
		
		# <w>と<i>の複合メソッドを追加
		Compiler.definline :wi
		def inline_wi(word)
			inline_i(inline_w(word))
		end

		# <b>と<idx>の複合メソッド
		Compiler.definline :bidx
		def inline_bidx(word)
			inline_b(word) + index(word)
		end
		
		# <w>と<idx>の複合メソッド
		Compiler.definline :widx
		def inline_widx(word)
			inline_idx(inline_w(word))
		end

		Compiler.definline :iimage
		def inline_iimage(arg)
			id, o = *arg.split(',', 2)
			inline_icon(id) # LaTeX以外はiconと同じ
		end

		Compiler.defsingle :halfblankline, 0
		def halfblankline
			# LaTeX以外は振る舞いなし
		end
	end

	class Builder
		prepend BuilderOverride
	end

	module LATEXBuilderOverride

		def builder_init_file
			super
			@image_txt = []
		end
		
		def image(lines, id, caption, metric = nil)
			super(lines, id, caption, metric)
			@image_txt.append "図: #{id}"
			@image_txt.append lines.push("\n")
			@image_txt.flatten!
		end

		def result
			if @book.config['text_in_imageblock']
				File.open("images.txt", mode = "a"){|f|
					@image_txt.each{|l| f.puts l}
				}
			end
			super
		end

		# イメージを伴うブロック
	  	Compiler.defblock :sideimagebox, 3
		def sideimagebox(lines, id, pos, w)
			blank
			puts "\\begin{wrapfigure}{#{pos}}{#{w}}"
			puts "\\vspace{-1\\baselineskip}"
			puts "\\includegraphics[keepaspectratio,width=#{w}]{./images/#{id}.png}"
			puts "\\end{wrapfigure}"
			puts split_paragraph(lines).join("\n")
			puts "\\vspace{1\\baselineskip}"
			blank
		end

		Compiler.defblock :fromauthor, 2
		def fromauthor(lines, id, pos)
			puts "\\vspace{1zw}"
			puts "\\begin{tcolorbox}[colback=gray!20,sharp corners,boxrule=0pt]"
			puts "\\noindent"
			if pos.eql?('l')
				puts "\\begin{minipage}[]{0.20\\textwidth}"
				puts "\\centering"
				puts "\\includegraphics[keepaspectratio,width=1.3cm]{./images/#{id}.png}"
				puts "\\end{minipage}\\hfill"
				puts "\\begin{minipage}[]{0.80\\textwidth}"
				puts "{\\gt {\\small"
				puts split_paragraph(lines).join("\n")
				puts "}}"
				puts "\\end{minipage}"
			else
				puts "\\begin{minipage}[]{0.80\\textwidth}"
				puts "{\\gt {\\small"
				puts split_paragraph(lines).join("\n")
				puts "}}"
				puts "\\end{minipage}\\hfill"
				puts "\\begin{minipage}[]{0.20\\textwidth}"
				puts "\\centering"
				puts "\\includegraphics[keepaspectratio,width=1.3cm]{./images/#{id}.png}"
				puts "\\end{minipage}"
			end
			puts "\\end{tcolorbox}"
			puts "\\vspace{1zw}"
		end

		# iimage: インラインイメージ。LaTeXのincludegraphicsのオプションを第2引数にそのまま書ける
		# iimage(画像ファイル, includegraphicsオプション)
		Compiler.definline :iimage
		def inline_iimage(arg)
			id, o = *arg.split(',', 2)
			"\\includegraphics[#{o}]{#{@chapter.image(id).path}}"
		end
		
		# 参考文献にthebibliography環境を利用する
		def bibpaper(lines, id, caption)
			puts %Q(\\bibitem{bib:#{id}} #{split_paragraph(lines).join("\n\n")})
			puts "\\vspace{0.2zw}"
		end

		Compiler.defsingle :halfblankline, 0
		def halfblankline
			puts "\\vspace{0.5zw}"
		end

                # 追加BEGIN
                def footnote(id, content)
                  if @book.config['footnotetext'] || @foottext[id]
                    # if @doc_status[:column]
                    if @doc_status[:column] || @doc_status[:minicolumn]
                      warn "//footnote[#{id}] is in the column block. It is recommended to move out of the column block.", location: location
                    end
                    #puts macro("footnotetext[#{@chapter.footnote(id).number}]", compile_inline(content.strip))
                    # 番号を自動に任せる
                    puts macro("footnotetext", compile_inline(content.strip))
                  end
                end

                def inline_fn(id)
                  if @book.config['footnotetext']
                    macro("footnotemark[#{@chapter.footnote(id).number}]", '')
                  # elsif @doc_status[:caption] || @doc_status[:table] || @doc_status[:column] || @doc_status[:dt]
                  elsif @doc_status[:caption] || @doc_status[:table] || @doc_status[:column] || @doc_status[:dt] || @doc_status[:minicolumn]
                    @foottext[id] = @chapter.footnote(id).number
                    macro('protect\\footnotemark', '')
                  else
                    macro('footnote', compile_inline(@chapter.footnote(id).content.strip))
                  end
                rescue KeyError
                  app_error "unknown footnote: #{id}"
                end
                # 追加END
	end
	
	class LATEXBuilder
		prepend LATEXBuilderOverride
	end
end
